#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2008-2011 Richard Dymond (rjdymond@gmail.com)
#
# This file is part of SkoolKit.
#
# SkoolKit is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# SkoolKit is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# SkoolKit. If not, see <http://www.gnu.org/licenses/>.

import sys
import time
import os.path
from skoolkit.skoolparser import SkoolParser, CASE_LOWER, CASE_UPPER, BASE_10, BASE_16
from skoolkit.skoolasm import SkoolWriter

DEF_INSTR_WIDTH = 23

def clock(quiet, prefix, operation, *args, **kwargs):
    go = time.time()
    result = operation(*args, **kwargs)
    stop = time.time()
    if not quiet:
        sys.stderr.write('%s (%0.2fs)\n' % (prefix, stop - go))
    return result

def parse_args(args):
    quiet = verbose = debug = crlf = tab = False
    case = base = None
    instr_width = DEF_INSTR_WIDTH
    asm_mode = 1
    fix_mode = 0
    warn = True
    create_labels = False
    files = []
    i = 0
    while i < len(args):
        arg = args[i]
        if arg == '-q':
            quiet = True
        elif arg == '-v':
            verbose = True
        elif arg == '-w':
            warn = False
        elif arg == '-e':
            warn = False
            debug = True
        elif arg == '-d':
            crlf = True
        elif arg == '-l':
            case = CASE_LOWER
        elif arg == '-u':
            case = CASE_UPPER
        elif arg == '-D':
            base = BASE_10
        elif arg == '-H':
            base = BASE_16
        elif arg[:2] == '-i':
            instr_width = int(arg[2:])
        elif arg[:2] == '-f':
            fix_mode = int(arg[2:])
        elif arg == '-r':
            asm_mode = 3
        elif arg == '-s':
            asm_mode = 2
        elif arg == '-t':
            tab = True
        elif arg == '-c':
            create_labels = True
        elif arg[0] == '-':
            print_usage()
        else:
            files.append(arg)
        i += 1

    if len(files) != 1:
        print_usage()
    parser_mode = (case, base, asm_mode, verbose, warn, fix_mode, create_labels)
    writer_mode = (crlf, tab, case == CASE_LOWER, instr_width, warn)
    return files[0], parser_mode, writer_mode, quiet, debug

def print_usage():
    sys.stderr.write('Usage: %s [options] <skoolfile>\n\n' % os.path.basename(sys.argv[0]))
    sys.stderr.write('Options:\n')
    sys.stderr.write("  -q   Be quiet (don't show timings)\n")
    sys.stderr.write('  -v   Be verbose (show instruction substitutions)\n')
    sys.stderr.write('  -w   Suppress warnings\n')
    sys.stderr.write('  -e   Write processed lines to stderr (implies -w)\n')
    sys.stderr.write('  -d   Use CR+LF to end lines\n')
    sys.stderr.write('  -t   Use tab to indent instructions (default indentation is 2 spaces)\n')
    sys.stderr.write("  -l   Write disassembly in lower case\n")
    sys.stderr.write("  -u   Write disassembly in upper case\n")
    sys.stderr.write("  -D   Write disassembly in decimal\n")
    sys.stderr.write("  -H   Write disassembly in hexadecimal\n")
    sys.stderr.write('  -iN  Set instruction field width to N (default=%i)\n' % DEF_INSTR_WIDTH)
    sys.stderr.write('  -fN  Apply fixes:\n')
    sys.stderr.write('         N=0: None (default)\n')
    sys.stderr.write('         N=1: @ofix only\n')
    sys.stderr.write('         N=2: @ofix and @bfix\n')
    sys.stderr.write('         N=3: @ofix, @bfix and @rfix (implies -r)\n')
    sys.stderr.write('  -c   Create default labels for unlabelled instructions\n')
    sys.stderr.write('  -s   Use safe substitutions (@ssub)\n')
    sys.stderr.write('  -r   Use relocatability substitutions too (@rsub) (implies -f1)\n')
    sys.exit(1)

def run(skoolfile, parser_mode, writer_mode, quiet, debug):
    # Create the parser
    case, base, asm_mode, verbose, warn, fix_mode, create_labels = parser_mode
    parser = clock(quiet, 'Parsed %s' % skoolfile, SkoolParser, skoolfile, case=case, base=base, asm_mode=asm_mode, verbose=verbose, fix_mode=fix_mode, create_labels=create_labels, debug=debug)

    # Write the ASM file
    skool_writer = SkoolWriter(parser, *writer_mode)
    clock(quiet, 'Wrote ASM to stdout', skool_writer.write)

def main():
    run(*parse_args(sys.argv[1:]))

if __name__ == '__main__':
    main()
