#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright 2009-2011 Richard Dymond (rjdymond@gmail.com)
#
# This file is part of SkoolKit.
#
# SkoolKit is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# SkoolKit is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# SkoolKit. If not, see <http://www.gnu.org/licenses/>.

import sys
from os.path import isfile, basename
from skoolkit.snaskool import SkoolWriter, Recipe, get_snapshot, parse_ctl, generate_ctl

def parse_args(args):
    ctlfile = None
    genctl = False
    ctl_hex = False
    asm_hex = False
    asm_lower = False
    start = 16384
    org = None
    text = False
    write_refs = True
    defb_size = 8
    defb_mod = 1
    defm_width = 66
    zfill = False
    files = []
    i = 0
    while i < len(args):
        arg = args[i]
        if arg == '-c':
            ctlfile = args[i + 1]
            if not isfile(ctlfile):
                sys.stderr.write('ERROR: %s: file not found\n' % ctlfile)
                sys.exit(1)
            i += 1
        elif arg == '-g':
            genctl = True
            ctlfile = args[i + 1]
            i += 1
        elif arg == '-h':
            ctl_hex = True
        elif arg == '-H':
            asm_hex = True
        elif arg == '-L':
            asm_lower = True
        elif arg == '-s':
            start = int(args[i + 1])
            i += 1
        elif arg == '-o':
            org = int(args[i + 1])
            i += 1
        elif arg == '-t':
            text = True
        elif arg == '-r':
            write_refs = False
        elif arg == '-n':
            defb_size = int(args[i + 1])
            i += 1
        elif arg == '-m':
            defb_mod = int(args[i + 1])
            i += 1
        elif arg == '-l':
            defm_width = int(args[i + 1])
            i += 1
        elif arg == '-z':
            zfill = True
        elif arg[0] == '-':
            print_usage()
        else:
            files.append(arg)
        i += 1

    if len(files) != 1 or files[0][-4:] not in ('.bin', '.sna', '.z80'):
        print_usage()
    snafile = files[0]
    if ctlfile is None:
        ctlfile = '%s.ctl' % snafile[:-4]
    return snafile, ctlfile, genctl, ctl_hex, asm_hex, asm_lower, start, org, text, write_refs, defb_size, defb_mod, zfill, defm_width

def print_usage():
    sys.stderr.write('Usage: %s [options] file.[sna|z80|bin]\n\n' % basename(sys.argv[0]))
    sys.stderr.write('Options:\n')
    sys.stderr.write('  -c FILE  Use FILE as the ctl file (default is file.ctl)\n')
    sys.stderr.write('  -g FILE  Generate a control file in FILE\n')
    sys.stderr.write('  -h       Write hexadecimal addresses in the generated control file\n')
    sys.stderr.write('  -H       Write hexadecimal addresses and operands in the disassembly\n')
    sys.stderr.write('  -L       Write the disassembly in lower case\n')
    sys.stderr.write('  -s ADDR  Specify the address at which to start disassembling\n')
    sys.stderr.write('  -o ADDR  Specify the origin address of file.bin (default: 65536 - length)\n')
    sys.stderr.write('  -t       Show ASCII text in the comment fields\n')
    sys.stderr.write('  -r       Suppress addition of comments listing entry point referrers\n')
    sys.stderr.write("  -n N     Set the max number of bytes per DEFB statement to N (default=8)\n")
    sys.stderr.write("  -m M     Group DEFB blocks by addresses that divide M\n")
    sys.stderr.write("  -z       Write bytes with leading zeroes in DEFB statements\n")
    sys.stderr.write("  -l L     Set the max number of characters per DEFM statement to L (default=66)\n")
    sys.exit(1)

def run(snafile, ctlfile, genctl, ctl_hex, asm_hex, asm_lower, start, org, text, write_refs, defb_size, defb_mod, zfill, defm_width):
    # Read the snapshot file
    if snafile[-4:] == '.bin':
        f = open(snafile, 'rb')
        ram = f.read()
        f.close()
        if org is None:
            org = 65536 - len(ram)
        snapshot = [0] * org + [ord(b) for b in ram]
        end = len(snapshot)
        snapshot += [0] * (65536 - end)
        ctls = {max((org, start)): 'c'}
        if end < 65536:
            ctls[end] = 'i'
        recipe = Recipe(ctls)
    else:
        snapshot = get_snapshot(snafile)
        recipe = Recipe({start: 'c'})

    # Pad out the end of the snapshot to avoid errors when an instruction
    # crosses the 64K boundary
    snapshot.extend((0, 0, 0))

    if genctl:
        # Analyse the snapshot and generate a ctl file...
        recipe = Recipe(generate_ctl(snapshot, ctlfile, ctl_hex, start))
    elif isfile(ctlfile):
        # ...or read the existing ctl file
        recipe = parse_ctl(ctlfile)

    # Print the skool/ASM file
    writer = SkoolWriter(snapshot, recipe, defb_size, defb_mod, zfill, defm_width, asm_hex, asm_lower)
    writer.write_skool(write_refs, text)

def main():
    run(*parse_args(sys.argv[1:]))

if __name__ == '__main__':
    main()
